import argparse
import zipfile
import os
import iio
import numpy as np
import glob
import time
import subprocess



def generate_noisy(cwd, iso=3200):
    # cwd = os.getcwd()
    print("Current workding directory: {}. Adding noise.".format(cwd))
    if iso==3200:
        a, b = 8.0034, -2043.51144
    elif iso==12800:
        a, b = 28.3015, 6307.62081
    else:
        print("ISO value invalid.")
    print("ISO value:",iso)
    print("a: {}, b: {}".format(a,b))

    seqlist = glob.glob(cwd+'/gt_iso'+str(iso)+'/*')
    for seq in seqlist:
        print("The ground truth sequences: ", seq)
        framelist = glob.glob(seq+'/*.tiff')

        # make directory for storing noisy sequences
        seq_noisy = seq.replace('gt','noisy')
        if not os.path.exists(seq_noisy):
            print("Creating new directory:\n   "+seq_noisy)
            os.makedirs(seq_noisy)

        for f in framelist:
            gt = iio.read(f)
            n, m, c = gt.shape

            # add noise to the GT
            noisy = gt + np.sqrt( (a*gt+b).clip(0, np.inf) )*np.random.randn(n,m,c)

            # save noisy frame    
            file_noisy = f.replace('gt','noisy')
            iio.write(file_noisy, noisy)


if __name__=='__main__':
    
    parser = argparse.ArgumentParser(description='unzip data and add noise if needed.')
    parser.add_argument('--zipdatapath', type=str, default='./demo_file/gt_flows.zip')
    parser.add_argument('--seqpath', type=str, default='/home/zheng/demo_scripts/demo_file/')
    # parser.add_argument('--savepath', type=str, default='/mnt/ddisk/zhengz/denoising_output')
    parser.add_argument('--iso', type=str, default='3200')
    # parser.add_argument('--VST', type=str, default='True')
    # parser.add_argument('--ratioVariance', action='store_true')


    args=parser.parse_args()
    
    with zipfile.ZipFile(args.zipdatapath, 'r') as files:
        # files.printdir()
        print("Extracting all files...")
        files.extractall(args.seqpath)

    # change to the data folder
    print("Changing to the data folder...")
    os.chdir(args.seqpath)
    cwd = os.getcwd()
    print("Current working directory:", os.getcwd())


    # Add noise to ground truth
    if not os.path.exists(cwd+'/noisy_iso'+str(args.iso)):
        print("Generate noisy video sequences.")
        generate_noisy(cwd, iso=int(args.iso))
    else:
        print("Noisy already data exists.")

    # Move the optical flow to the right directory
    # orig_path = cwd+'/flow/'
    # print("The path of the optical flows: ",orig_path)
    # des_path = orig_path+'noisy_iso3200/tvl1/noisyinputs/'

    # if os.path.exists(orig_path):
    #     filelist = os.listdir(orig_path)
    #     # print(filelist)
    #     for file in filelist:
    #         if not 'noisy' in file:
    #             print(des_path+file)
    #             if not os.path.exists(des_path+file):
    #                 os.makedirs(des_path+file)
    #                 os.rename(orig_path+file, des_path+file)
    # else:
    #     print('Path invalid!')